%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project: Spatially Varying Coefficient Models
% Simulation Example: Example 1 in Mu, Wang and Wang (2017)
% Function: svcm_est.m (Function for Bivariate Spline Estimation)
% Author: GuanNan Wang, JingRu Mu and Lily Wang
%
% Inputs: 
% (1) X: Explanatory Variables
% (2) Y: Response Variable
% (3) B: Berstein Spline Basis
% (4) Q2: QR decomposition of the transpose of smoothness condition matrix
% (5) K: Energy Matrix
% (6) lambda: Tuning Parameter
% (7) method: 'bpst' or 'bst' ('Bivariate Penalized Spline' or 'Bivariate Spline') 
%
% Outputs: 
% (1) beta_hat: Estimated Coefficient Function
% (2) Y_hat: Fitted Value for Response Variable
% (3) sse: Sum of Squared Error
% (4) lambda_c: Tuning Parameter Chosen by CV/GCV
% (5) t: Computation Time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [beta_hat,gamma_hat,Y_hat,sse,lambda_c,t]=...
    svcm_est(X,Y,B,Q2,K,lambda,method)
n=length(Y);
np=size(X,2);
 
% Bivariate Penalized Spline Estimation
if(strcmp(method,'bpst'))
    % grid search for best penalty parameter lambda
    lambda=repmat(lambda',1,np);
    nl=size(lambda,1);

    t1=cputime;
    % 5-fold Cross-Validation;
    indp=randperm(n);
    sse=zeros(nl,1);
    for(il=1:nl)
        for(ik=1:5)
            nk=floor(n/5);
            if(ik<5)
                indk=indp(((ik-1)*nk+1):(ik*nk));
            end;
            if(ik==5)
                indk=indp(((ik-1)*nk+1):n);
            end;
            indnk=setdiff(indp,indk);
            Yk=Y(indk,:);
            Ynk=Y(indnk,:);
            Xk=X(indk,:);
            Xnk=X(indnk,:);
            Bk=B(indk,:);
            Bnk=B(indnk,:);
            [betank,gamma,Ynkhat,ssenk]=...
                svcm_fit(Xnk,Ynk,Bnk,Q2,K,lambda(il,:),method); 
            betak=[];
            for (ip=1:np)
                gammap=gamma(:,ip);
                betap=Bk*gammap;
                betak=[betak betap];
            end;
            Ykhat=sum(Xk.*betak,2);
            sse(il)=sse(il)+sum((Yk-Ykhat).^2);
        end;
    end;
    %plot(sse);
    [J,j]=min(sse); 
    lambda_c=lambda(j,:);
    [beta_hat,gamma_hat,Y_hat,sse]=...
        svcm_fit(X,Y,B,Q2,K,lambda_c,method);
	t=cputime-t1;
end;
% Bivariate Spline Estimation
if(strcmp(method,'bst'))
    lambda_c=0;
	t2=cputime;
	[beta_hat,gamma_hat,Y_hat,sse]=...
        svcm_fit(X,Y,B,Q2,K,lambda_c,method);
    t=cputime-t2;
end;
end
