%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project: Spatially Varying Coefficient Models
% Simulation Example: Example 1 in Mu, Wang and Wang (2017)
% Function: svcm_fit.m (Least Square Fitting)
% Author: GuanNan Wang, JingRu Mu and Lily Wang
%
% Inputs: 
% (1) X: Explanatory Variables
% (2) Y: Response Variable
% (3) B: Berstein Spline Basis
% (4) Q2: QR decomposition of the transpose of smoothness condition matrix
% (5) K: Energy Matrix
% (6) lambda: Tuning Parameter
% (7) method: 'bpst' or 'bst' ('Bivariate Penalized Spline' or 'Bivariate Spline') 
%
% Outputs: 
% (1) beta: Coefficient Function
% (2) Yhat: Fitted Value
% (3) sse: Sum of Squared Error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [beta,gamma,Yhat,sse]=svcm_fit(X,Y,B,Q2,K,lambda,method)
n=length(Y);
np=size(X,2);
nq=size(Q2,2);

BQ2=B*Q2;
Hmtx=0;
Svec=0;
for(i=1:n)
    Xi=X(i,:);
    Yi=Y(i);
    BiQ2=BQ2(i,:);
    XiBiQ2=kron(Xi,BiQ2);
    Hmtx=Hmtx+XiBiQ2'*XiBiQ2;
    Svec=Svec+XiBiQ2'*Yi;
end;
if(strcmp(method,'bpst'))
    Lam=diag(lambda);
    Dlam=kron(Lam,Q2'*K*Q2);
    theta=inv(Hmtx+Dlam)*Svec;
end;
if(strcmp(method,'bst'))
    theta=inv(Hmtx)*Svec;
end;
thetamtx=reshape(theta,nq,np);
gamma=Q2*thetamtx;
beta=B*Q2*thetamtx;
Yhat=sum(X.*beta,2);
sse=sum((Y-Yhat).^2);
end