%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project: Spatially Varying Coefficient Models
% Simulation Example in Mu, Wang and Wang (2017)
% Function: svcm_test.m (Function for Test of Stationarity)
% Author: GuanNan Wang, JingRu Mu and Lily Wang
%
% Inputs: 
% (1) X: Explanatory Variables
% (2) Y: Response Variable
% (3) B: Berstein Spline Basis
% (4) Q2: QR decomposition of the transpose of smoothness condition matrix
% (5) K: Energy Matrix
% (6) lambda: Tuning Parameter
% (7) nB: Number of Resampling
% (8) test: 'global' or 'individual' ('Global Test of Stationarity' or 'Individual Test of Stationarity'
%
% Outputs: 
% (1) pvalue: P-value of the Test
% (2) t: Computation Time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pvalue t]=svcm_test(X,Y,B,Q2,K,lambda,nB,test)
n=length(Y);
np=size(X,2);

[beta_bpst,gamma_bpst,Y_bpst,sse_bpst,lambda_c,t]=...
    svcm_est(X,Y,B,Q2,K,lambda,'bpst');

% Global Test of Stationarity;
if(strcmp(test,'global'))
    E=Y-Y_bpst;
    E=E-mean(E);
    beta_ls=inv(X'*X)*X'*Y;
	Y_ls=X*beta_ls;
	sse_ls=sum((Y-Y_ls).^2);
	Tobs=(sse_ls-sse_bpst)/sse_bpst;
    T_boot=[];
	t1=cputime;
    for(ib=1:nB)
        indp=randsample(n,n,true);
        Eb=E(indp);
        Yb=Y_ls+Eb;
        [betab1,gammab1,Ybhat1,sseb1]=svcm_fit(X,Yb,B,Q2,K,lambda_c,'bpst'); 
        betab2=inv(X'*X)*X'*Yb;
        Ybhat2=X*betab2;
        sseb2=sum((Yb-Ybhat2).^2);
        T_boot=[T_boot; (sseb2-sseb1)/sseb1];
    end;
	pvalue=mean(T_boot>=Tobs);
	t=cputime-t1;
end;

% Individual Test of Stationarity;
if(strcmp(test,'individual'))
    V_bpst=var(beta_bpst);
    V_boot=[];
    pvalue=[];
    t2=cputime;
    for(ib=1:nB)
        indp=randsample(n,n,false);
        Bp=B(indp,:); 
        [beta,gamma,Yhat,sse]=svcm_fit(X,Y,Bp,Q2,K,lambda_c,'bpst'); 
        temp=var(beta);
        V_boot=[V_boot; temp];
    end;
    
    np=length(V_bpst);
    for(ip=1:np)
        pvalue=[pvalue mean(V_boot(:,ip)>=V_bpst(ip))];
    end;
    t=cputime-t2;
end;
end
    



